@@ -1,11 +1,11 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1999-2005 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1999-2008 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: CopyJob.cc,v 1.33 2006/02/06 11:03:58 lav Exp $ */
+/* $Id: CopyJob.cc,v 1.43 2008/11/27 05:56:10 lav Exp $ */
 
 #include <config.h>
 #include "CopyJob.h"
@@ -27,6 +27,8 @@
 #include "misc.h"
 #include "url.h"
 
+#define waiting_num waiting.count()
+
 int CopyJob::Do()
 {
    if(!fg_data)
@@ -35,7 +37,7 @@ int CopyJob::Do()
       return STALL;
    if(c->Error())
    {
-      eprintf("%s: %s\n",op,c->ErrorText());
+      eprintf("%s: %s\n",op.get(),c->ErrorText());
       done=true;
       return MOVED;
    }
@@ -56,10 +58,6 @@ int CopyJob::Do()
    }
    return STALL;
 }
-int CopyJob::Done()
-{
-   return done;
-}
 int CopyJob::ExitCode()
 {
    if(c->Error())
@@ -85,16 +83,11 @@ const char *CopyJob::Status(const StatusLine *s, bool base)
    if(c->Done() || c->Error())
       return "";
 
-   static char *buf = 0;
-   xfree(buf);
-
    const char *name=SqueezeName(s->GetWidthDelayed()-50, base);
-   buf = xasprintf(COPY_STATUS);
-
-   return buf;
+   return xstring::format(COPY_STATUS);
 }
 
-void CopyJob::ShowRunStatus(StatusLine *s)
+void CopyJob::ShowRunStatus(const SMTaskRef<StatusLine>& s)
 {
    if(no_status)
       return;
@@ -136,22 +129,16 @@ int CopyJob::AcceptSig(int sig)
 
 void CopyJob::SetDispName()
 {
-   xfree(dispname);
-   dispname=0;
-
    ParsedURL url(name,true);
    if(url.proto)
-      dispname = xstrdup(url.path);
+      dispname.set(url.path);
    else
-      dispname = xstrdup(name);
+      dispname.set(name);
 }
 
 CopyJob::CopyJob(FileCopy *c1,const char *name1,const char *op1)
+   : c(c1), name(name1), op(op1)
 {
-   c=c1;
-   dispname=0;
-   name=xstrdup(name1);
-   op=xstrdup(op1);
    done=false;
    no_status=false;
    no_status_on_write=false;
@@ -159,14 +146,6 @@ CopyJob::CopyJob(FileCopy *c1,const char *name1,const char *op1)
    SetDispName();
 }
 
-CopyJob::~CopyJob()
-{
-   Delete(c);
-   xfree(name);
-   xfree(dispname);
-   xfree(op);
-}
-
 const char *CopyJob::FormatBytesTimeRate(off_t bytes,double time_spent)
 {
    static char buf[256];
@@ -209,14 +188,11 @@ CopyJobEnv::CopyJobEnv(FileAccess *s,ArgV *a,bool cont1)
    no_status=false;
    cont=cont1;
    ascii=false;
-   cwd=xgetcwd();
+   cwd.set_allocated(xgetcwd());
 }
 CopyJobEnv::~CopyJobEnv()
 {
    SetCopier(0,0);
-   if(args)
-      delete args;
-   xfree(cwd);
 }
 int CopyJobEnv::Do()
 {
@@ -243,6 +219,11 @@ int CopyJobEnv::Do()
    count++;
    bytes+=j->GetBytesCount();
    time_spent+=j->GetTimeSpent();
+   Delete(j);
+   if(cp==j)
+      cp=0;
+   if(waiting_num>0 && cp==0)
+      cp=(CopyJob*)waiting[0];
    return MOVED;
 }
 void CopyJobEnv::AddCopier(FileCopy *c,const char *n)
@@ -251,8 +232,7 @@ void CopyJobEnv::AddCopier(FileCopy *c,const char *n)
       return;
    if(ascii)
       c->Ascii();
-   cp=new CopyJob(c,n,op);
-   cp->SetParentFg(this);
+   cp=cj_new?cj_new->New(c,n,op):new CopyJob(c,n,op);
    AddWaiting(cp);
 }
 void CopyJobEnv::SetCopier(FileCopy *c,const char *n)
