@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: DummyProto.h,v 1.8 2001/12/05 15:03:41 lav Exp $ */
+/* $Id: DummyProto.h,v 1.11 2008/11/27 05:56:11 lav Exp $ */
 
 #ifndef DUMMYPROTO_H
 #define DUMMYPROTO_H
@@ -30,8 +30,8 @@ class DummyProto : public FileAccess
 public:
    int Do();
    int Done();
-   const char *GetProto();
-   FileAccess *Clone();
+   const char *GetProto() const;
+   FileAccess *Clone() const;
    int Read(void *buf,int size);
    int Write(const void *buf,int size);
    int StoreStatus();
@@ -43,13 +43,12 @@ public:
 
 class DummyNoProto : public DummyProto
 {
-   char *proto;
+   xstring_c proto;
 public:
-   DummyNoProto(const char *p);
-   ~DummyNoProto();
+   DummyNoProto(const char *p) : proto(p) {}
 
-   const char *GetProto();
-   FileAccess *Clone();
+   const char *GetProto() const;
+   FileAccess *Clone() const;
    const char *StrError(int err);
 };
 
