@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileAccess.cc,v 1.155 2008/05/16 08:44:57 lav Exp $ */
+/* $Id: FileAccess.cc,v 1.159 2008/11/27 05:56:11 lav Exp $ */
 
 #include <config.h>
 
@@ -159,7 +159,7 @@ void FileAccess::CloseOnExec(int fd)
 void  FileAccess::Open(const char *fn,int mode,off_t offs)
 {
 #ifdef OPEN_DEBUG
-   printf("FA::Open(%s,%d)\n",fn?fn:"NULL",mode);
+   printf("%p->FA::Open(%s,%d)\n",this,fn?fn:"NULL",mode);
 #endif
    if(IsOpen())
       Close();
@@ -944,13 +944,14 @@ ListInfo::ListInfo(FileAccess *s,const char *p)
    }
 }
 
-ListInfo::~ListInfo()
+void ListInfo::PrepareToDie()
 {
    if(session)
       session->Close();
    if(session && saved_cwd)
       session->SetCwd(saved_cwd);
 }
+ListInfo::~ListInfo() {}
 
 
 // Path implementation
@@ -1157,3 +1158,11 @@ const FileAccessRef& FileAccessRef::operator=(FileAccess *p)
    ptr=SMTask::MakeRef(p);
    return *this;
 }
+
+// hook-up gnulib...
+CDECL_BEGIN
+#include "md5.h"
+#include "glob.h"
+CDECL_END
+void *_md5_hook=(void*)md5_init_ctx;
+void *_glob_hook=(void*)glob;
