@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,24 +18,13 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileAccess.h,v 1.107 2008/05/16 08:44:57 lav Exp $ */
+/* $Id: FileAccess.h,v 1.110 2008/11/27 05:56:11 lav Exp $ */
 
 #ifndef FILEACCESS_H
 #define FILEACCESS_H
 
-#include <time.h>
 #include <sys/types.h>
-
-#ifdef HAVE_SYS_STROPTS_H
-# include <sys/stropts.h>
-#endif
-
-#ifdef HAVE_SYS_POLL_H
-# include <sys/poll.h>
-#else
-# include <poll.h>
-#endif
-#include <assert.h>
+#include <time.h>
 
 #include "SMTask.h"
 #include "trio.h"
@@ -497,6 +486,8 @@ protected:
    unsigned need;
    bool follow_symlinks;
 
+   void PrepareToDie();
+
 public:
    ListInfo(FileAccess *session,const char *path);
    ~ListInfo();
