@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -25,22 +25,14 @@
 
 class FileFeeder : public CmdFeeder
 {
-   FDStream *in;
+   Ref<FDStream> in;
    enum { buffer_size=0x1000 };
    char buffer[buffer_size];
-   FgData *fg_data;
+   Ref<FgData> fg_data;
 public:
    const char *NextCmd(CmdExec *exec,const char *prompt);
-   FileFeeder(FDStream *in)
-   {
-      this->in=in;
-      fg_data=0;
-   }
-   virtual ~FileFeeder()
-   {
-      delete fg_data;
-      delete in;
-   }
+   FileFeeder(FDStream *in);
+   ~FileFeeder();
    void Fg() { if(fg_data) fg_data->Fg(); }
    void Bg() { if(fg_data) fg_data->Bg(); }
 };
