@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: LocalAccess.h,v 1.12 2006/06/13 14:35:38 lav Exp $ */
+/* $Id: LocalAccess.h,v 1.15 2008/11/27 05:56:21 lav Exp $ */
 
 #ifndef LOCALACCESS_H
 #define LOCALACCESS_H
@@ -28,7 +28,7 @@
 
 class LocalAccess : public FileAccess
 {
-   FDStream *stream;
+   Ref<FDStream> stream;
    bool done;
    void errno_handle();
    void fill_array_info();
@@ -37,7 +37,6 @@ public:
    void Init();
    LocalAccess();
    LocalAccess(const LocalAccess *);
-   ~LocalAccess();
 
    void Connect(const char *host,const char *port) {}
    void AnonymousLogin() {}
@@ -64,23 +63,4 @@ public:
    DirList  *MakeDirList(ArgV *a);
 };
 
-class LocalListInfo : public ListInfo
-{
-public:
-   LocalListInfo(FileAccess *s,const char *d) : ListInfo(s,d) {}
-   const char *Status() { return "..."; }
-   int Do();
-};
-
-class LocalDirList : public DirList
-{
-   IOBuffer *ubuf;
-   FgData *fg_data;
-public:
-   LocalDirList(ArgV *a,const char *cwd);
-   ~LocalDirList();
-   const char *Status() { return "..."; }
-   int Do();
-};
-
 #endif//LOCALACCESS_H
