@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,10 +18,11 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Resolver.cc,v 1.78 2007/10/08 13:29:14 lav Exp $ */
+/* $Id: Resolver.cc,v 1.82 2008/11/27 05:56:26 lav Exp $ */
 
 #include <config.h>
 
+#include <stddef.h>
 #include "Resolver.h"
 #include "SignalHook.h"
 #include <errno.h>
@@ -30,6 +31,7 @@
 #include <time.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#include <netdb.h>
 #include <ctype.h>
 #include <fcntl.h>
 
@@ -266,7 +268,19 @@ int   Resolver::Do()
    if(!cache)
       cache=new ResolverCache;
    cache->Add(hostname,portname,defport,service,proto,addr.get(),addr.count());
-   FA::LogNote(4,plural("%d address$|es$ found",addr.count()),addr.count());
+
+   xstring report;
+   report.set(xstring::format(plural("%d address$|es$ found",addr.count()),addr.count()));
+   if(addr.count()>0) {
+      report.append(": ");
+      for(int i=0; i<addr.count(); i++) {
+	 report.append(addr[i].address());
+	 if(i<addr.count()-1)
+	    report.append(", ");
+      }
+   }
+   FA::LogNote(4,"%s",report.get());
+
    return MOVED;
 }
 
@@ -310,6 +324,8 @@ void Resolver::AddAddress(int family,const char *address,int len)
    default:
       return;
    }
+   if(addr.count()>0 && addr.last()==add)
+      return;
    addr.append(add);
 }
 
