@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,29 +18,16 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Resolver.h,v 1.24 2007/10/08 13:29:15 lav Exp $ */
+/* $Id: Resolver.h,v 1.26 2008/11/27 05:56:27 lav Exp $ */
 
 #ifndef RESOLVER_H
 #define RESOLVER_H
 
 #include "ProcWait.h"
-#include <netinet/in.h>
-#include <netdb.h>
-#include <string.h>
-#include <sys/types.h>
-#include <sys/socket.h>
 #include "buffer.h"
 #include "xarray.h"
 #include "Cache.h"
-
-union sockaddr_u
-{
-	struct sockaddr		sa;
-	struct sockaddr_in	in;
-#if INET6
-	struct sockaddr_in6	in6;
-#endif
-};
+#include "network.h"
 
 class Resolver : public SMTask
 {
