@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,19 +18,18 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SFtp.h,v 1.26 2008/01/21 14:04:31 lav Exp $ */
+/* $Id: SFtp.h,v 1.29 2008/11/27 05:56:27 lav Exp $ */
 
 #ifndef SFTP_H
 #define SFTP_H
 
-#include "NetAccess.h"
+#include "SSH_Access.h"
 #include "StatusLine.h"
-#include "PtyShell.h"
 #include <sys/types.h>
 #include <sys/stat.h>
 #include "FileSet.h"
 
-class SFtp : public NetAccess
+class SFtp : public SSH_Access
 {
    int	 protocol_version;
 
@@ -219,8 +218,6 @@ private:
    };
 
    state_t state;
-   bool received_greeting;
-   int  password_sent;
    unsigned ssh_id;
    xstring handle;
 
@@ -229,10 +226,6 @@ private:
    void	 SendMethod();
    void	 SendArrayInfoRequests();
 
-   SMTaskRef<IOBuffer> send_buf;
-   SMTaskRef<IOBuffer> recv_buf;
-   SMTaskRef<IOBuffer> pty_send_buf;
-   SMTaskRef<IOBuffer> pty_recv_buf;
    Ref<DirectedBuffer> send_translate;
    Ref<DirectedBuffer> recv_translate;
 
@@ -240,8 +233,6 @@ private:
    Ref<FileSet> file_set;
    Timer flush_timer;
 
-   Ref<PtyShell> ssh;
-
    void Disconnect();
    int IsConnected() const
       {
@@ -268,7 +259,7 @@ public:
       static bool is_valid_reply(int p)
       {
 	 return p==SSH_FXP_VERSION
-	    || p>=101 && p<=105
+	    || (p>=101 && p<=105)
 	    || p==SSH_FXP_EXTENDED_REPLY;
       }
    protected:
@@ -777,7 +768,7 @@ public:
    void Cleanup();
    void CleanupThis();
 
-   FileSet *GetFileSet() { FileSet *fset=file_set.borrow(); return fset?fset:new FileSet; }
+   FileSet *GetFileSet();
 };
 
 class SFtpDirList : public DirList
