@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -53,6 +53,7 @@ protected:
    // SuspendInternal and ResumeInternal usually suspend and resume slave tasks
    virtual void SuspendInternal() {}
    virtual void ResumeInternal() {}
+   virtual void PrepareToDie() {}  // it is called from Delete no matter of running and ref_count
 
 public:
    static void Block(int fd,int mask) { block.AddFD(fd,mask); }
@@ -81,7 +82,7 @@ public:
    SMTask();
    virtual ~SMTask();
 
-   void DeleteLater() { deleting=true; }
+   void DeleteLater();
    static void Delete(SMTask *);
    void IncRefCount() { ref_count++; }
    void DecRefCount() { if(ref_count>0) ref_count--; }
