@@ -1,11 +1,11 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1998-2005 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1998-2008 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,10 +18,11 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SleepJob.cc,v 1.34 2007/11/03 15:29:35 lav Exp $ */
+/* $Id: SleepJob.cc,v 1.40 2008/11/27 05:56:29 lav Exp $ */
 
 #include <config.h>
 #include <ctype.h>
+#include <stddef.h>
 #include "SleepJob.h"
 #include "CmdExec.h"
 #include "misc.h"
@@ -37,13 +38,11 @@ SleepJob::SleepJob(const TimeInterval &when,FileAccess *s,LocalDirectory *cwd,ch
    weak=false;
    repeat_count=0;
    max_repeat_count=0;
-   exec=0;
    continue_code=-1;
    break_code=-1;
 }
 SleepJob::~SleepJob()
 {
-   Delete(exec);
 }
 
 int SleepJob::Do()
@@ -78,14 +77,12 @@ int SleepJob::Do()
 	 if(!exec)
 	 {
 	    exec=new CmdExec(session.borrow(),saved_cwd.borrow());
-	    exec->SetParentFg(this);
 	    exec->AllocJobno();
 	    exec->cmdline.vset("(",cmd.get(),")",NULL);
 	 }
 	 exec->FeedCmd(cmd);
 	 exec->FeedCmd("\n");
-	 AddWaiting(exec);
-	 exec=0;
+	 AddWaiting(exec.borrow());
 	 return MOVED;
       }
       done=true;
@@ -239,17 +236,15 @@ Job *cmd_at(CmdExec *parent)
 
    xstring_ca date(args->Combine(1));
    date.truncate(date_len);
-   time_t now=SMTask::now;
-   time_t when=get_date(date,&now);
 
-   if(when==0 || when==(time_t)-1)
+   struct timespec ts;
+   if(!get_date(&ts,date,0))
    {
-      const char *e=get_date_error();
-      eprintf("%s: %s\n",args->a0(),e?e:"unknown parse error");
+      eprintf("%s: %s\n",args->a0(),"date parse error");
       return 0;
    }
-
-   if(when<now)
+   time_t when=ts.tv_sec;
+   if(when<SMTask::now)
       when+=86400;
 
    char *cmd=0;
