@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: SleepJob.h,v 1.14 2007/11/03 15:29:35 lav Exp $ */
+/* $Id: SleepJob.h,v 1.16 2008/11/27 05:56:29 lav Exp $ */
 
 #ifndef SLEEPJOB_H
 #define SLEEPJOB_H
@@ -34,7 +34,7 @@ class SleepJob : public SessionJob, public Timer
    int exit_code;
    bool done;
    Ref<LocalDirectory> saved_cwd;
-   CmdExec *exec;
+   JobRef<CmdExec> exec;
    bool repeat;
    bool weak;  // terminate on `exit bg'.
    int repeat_count;
