@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,20 +18,21 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Speedometer.h,v 1.3 2000/06/09 00:17:57 lav Exp $ */
+/* $Id: Speedometer.h,v 1.6 2008/11/27 05:56:29 lav Exp $ */
 
 #ifndef SPEEDOMETER_H
 #define SPEEDOMETER_H
 
 #include "SMTask.h"
+#include "ResMgr.h"
 
-class Speedometer : public SMTask
+class Speedometer : public SMTask, public ResClient
 {
    int period;
    float rate;
-   time_t last_second;
-   time_t last_bytes;
-   time_t start;
+   Time last_second;
+   Time last_bytes;
+   Time start;
    bool terse;
    const char *period_resource;
    static char buf_eta[];
