@@ -1,11 +1,11 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2006 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2008 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: lftp.cc,v 1.80 2008/05/22 13:20:40 lav Exp $ */
+/* $Id: lftp.cc,v 1.82 2008/11/27 05:56:38 lav Exp $ */
 
 #include <config.h>
 
@@ -138,8 +138,7 @@ public:
 
 	 if(ctty) // controlling terminal
 	 {
-	    pid_t term_pg=tcgetpgrp(0);
-	    if(term_pg!=(pid_t)-1 && getpgrp()!=term_pg)
+	    if(!in_foreground_pgrp())
 	    {
 	       // looks like we are in background. Can't read from tty
 	       exec->Timeout(500);
@@ -264,7 +263,7 @@ CMD(history)
    if(const char *arg = args->getcurr()) {
       if(!strcasecmp(arg, "all"))
 	 cnt = -1;
-      else if(isdigit(arg[0]))
+      else if(isdigit((unsigned char)arg[0]))
 	 cnt = atoi(arg);
       else {
 	 eprintf(_("%s: %s - not a number\n"), args->a0(), args->getcurr());
