@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: mgetJob.cc,v 1.25 2008/05/22 13:20:43 lav Exp $ */
+/* $Id: mgetJob.cc,v 1.28 2008/11/27 05:56:40 lav Exp $ */
 
 #include <config.h>
 #include <sys/types.h>
@@ -149,7 +149,6 @@ int mgetJob::Do()
       if(mkdir_job->Done())
       {
 	 RemoveWaiting(mkdir_job);
-	 Delete(mkdir_job);
 	 mkdir_job=0;
       }
       else
@@ -162,21 +161,18 @@ int mgetJob::Do()
    if(!rg)
    {
    next:
-      if(rg) { delete rg; rg=0; }
+      if(rg)
+	 rg=0;
       const char *p=m_args->getnext();
       if(!p)
       {
-	 delete m_args;
 	 m_args=0;
 	 if(mkdir_args)
 	 {
-	    mkdir_job=new mkdirJob(Clone(),mkdir_args);
+	    mkdir_job=new mkdirJob(Clone(),mkdir_args.borrow());
 	    mkdir_job->BeQuiet();
 	    AddWaiting(mkdir_job);
-	    mkdir_job->SetParentFg(this);
 	    mkdir_job->cmdline.set_allocated(mkdir_args->Combine());
-	    // don't delete mkdir_args; -- mkdirJob does it
-	    mkdir_args=0;
 	 }
 	 return MOVED;
       }
@@ -218,7 +214,4 @@ int mgetJob::Do()
 
 mgetJob::~mgetJob()
 {
-   delete rg;
-   delete m_args;
-   delete mkdir_args;
 }
