@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: misc.cc,v 1.94 2008/05/22 13:20:44 lav Exp $ */
+/* $Id: misc.cc,v 1.97 2008/11/27 05:56:41 lav Exp $ */
 
 #include <config.h>
 #include "xmalloc.h"
@@ -32,6 +32,7 @@
 #include <time.h>
 #include <ctype.h>
 #include <sys/ioctl.h>
+#include <fcntl.h>
 
 #ifdef HAVE_TERMIOS_H
 #include <termios.h>
@@ -52,10 +53,6 @@ CDECL_END
 #include <mbswidth.h>
 #include "strftime.h"
 
-#if HAVE_UNSETENV && !HAVE_DECL_UNSETENV
-CDECL void unsetenv(const char *name);
-#endif
-
 const char *dir_file(const char *dir,const char *file)
 {
    if(dir==0 || dir[0]==0)
@@ -488,30 +485,6 @@ mktime_from_utc (const struct tm *t)
    return (tl <= tb ? (tl + (tl - tb)) : (tl - (tb - tl)));
 }
 
-#ifndef HAVE_UNSETENV
-# ifdef HAVE_ENVIRON
-extern char **environ;
-void unsetenv(const char *env)
-{
-   char **scan=environ;
-   if(!scan)
-      return;
-   int env_len=strlen(env);
-   while(*scan)
-   {
-      if(!strncmp(*scan,env,env_len) && (*scan)[env_len]=='=')
-	 break;
-      scan++;
-   }
-   while(*scan)
-   {
-      *scan=*(scan+1);
-      scan++;
-   }
-}
-# endif
-#endif
-
 static void set_tz(const char *tz)
 {
    static char *put_tz;
@@ -597,7 +570,7 @@ char *Subst(const char *txt, const subst_t *s)
       if(ch=='\\' && *txt && *txt!='\\')
       {
 	 ch=*txt++;
-	 if(isdigit(ch) && ch != '8' && ch != '9') {
+	 if(ch >= '0' && ch < '8') {
 	    unsigned len;
 	    unsigned code;
 	    txt--;
@@ -918,3 +891,20 @@ void remove_tags(char *buf)
       buf=less;
    }
 }
+
+bool in_foreground_pgrp()
+{
+   static int tty_fd;
+   if(tty_fd==-1)
+      return true;
+   pid_t pg=tcgetpgrp(tty_fd);
+   if(pg==(pid_t)-1 && !isatty(tty_fd)) {
+      tty_fd=open("/dev/tty",O_RDONLY);
+      if(tty_fd==-1)
+	 return true;
+      pg=tcgetpgrp(tty_fd);
+   }
+   if(pg==(pid_t)-1 || pg==getpgrp())
+      return true;
+   return false;
+}
