@@ -1,11 +1,11 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2001 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2005 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: misc.h,v 1.36 2005/03/05 06:53:17 lav Exp $ */
+/* $Id: misc.h,v 1.50 2008/11/27 05:56:41 lav Exp $ */
 
 #ifndef MISC_H
 #define MISC_H
@@ -31,15 +31,15 @@
 #endif
 #include <stdarg.h>
 
+class xstring;
+
 // expands tilde; returns pointer to static data
 const char *expand_home_relative(const char *);
 
 // returns ptr to last path element
 const char *basename_ptr(const char *);
-static inline
-char *basename_ptr(char *f)
-{
-   return (char*)basename_ptr((const char *)f);
+static inline char *basename_ptr(char *f) {
+   return const_cast<char*>(basename_ptr(const_cast<const char *>(f)));
 }
 
 // glues file to dir; returns pointer to static storage
@@ -62,6 +62,10 @@ void  truncate_file_tree(const char *dir);
 /* returns file descriptor terminal width; -1 on error, 0 on N/A */
 int fd_width(int fd);
 
+/* returns true if current pgrp is the foreground pgrp of controlling tty
+ * or if the fg pgrp is unknown */
+bool in_foreground_pgrp();
+
 // returns malloc'ed cwd no matter how long it is
 // returns 0 on error.
 char *xgetcwd();
@@ -70,10 +74,6 @@ int percent(off_t offset,off_t size);
 
 #define find_char(buf,len,ch) ((const char *)memchr(buf,ch,len))
 
-#define MINUTE (60)
-#define HOUR   (60*MINUTE)
-#define DAY    (24*HOUR)
-
 extern const char month_names[][4];
 
 int parse_month(const char *);
@@ -96,9 +96,6 @@ struct subst_t {
  * with corresponding character. Returns allocated buffer to be free'd */
 char *Subst(const char *txt, const subst_t *s);
 
-char **tokenize(const char *str, int *argc = NULL);
-void tokenize_free(char **argv);
-
 /* uses gettimeofday if available */
 void xgettimeofday(time_t *sec, int *usec);
 
@@ -109,9 +106,9 @@ char *xstrftime(const char *format, const struct tm *tm);
 char *xvasprintf(const char *format, va_list ap);
 char *xasprintf(const char *format, ...);
 
-char *dirname_alloc(const char *fn);
-char *dirname_modify(char *fn);
-char *strip_trailing_slashes(char *fn);
+void strip_trailing_slashes(xstring& fn);
+xstring& dirname_modify(xstring& fn);
+xstring& dirname(const char *path);  // returns a tmp
 
 /* returns last character of string or \0 if string is empty */
 char last_char(const char *str);
@@ -119,4 +116,35 @@ char last_char(const char *str);
 int  base64_length (int len);
 void base64_encode (const char *s, char *store, int length);
 
+bool temporary_network_error(int e);
+
+CDECL const char *get_lftp_home();
+
+const char *memrchr(const char *buf,char c,size_t len);
+static inline char *memrchr(char *buf,char c,size_t len) {
+   return const_cast<char*>(memrchr(const_cast<const char*>(buf),c,len));
+}
+
+const char *shell_encode(const char *);
+void remove_tags(char *buf);
+
+#define ListAdd(type,chain,this,next)  \
+do {				       \
+   this->next=chain;		       \
+   chain=this;			       \
+} while(0)
+
+#define ListDel(type,chain,this,next)			 \
+do {							 \
+   for(type **scan=&chain; *scan; scan=&scan[0]->next)	 \
+      if(*scan==this)					 \
+      {							 \
+	 *scan=scan[0]->next;				 \
+	 break;						 \
+      }							 \
+} while(0)
+
+#define ListScan(type,chain,next) \
+   for(type *scan=chain; scan; scan=scan->next)
+
 #endif // MISC_H
