@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: rmJob.cc,v 1.9 2006/02/06 10:57:32 lav Exp $ */
+/* $Id: rmJob.cc,v 1.11 2008/11/27 05:56:44 lav Exp $ */
 
 #include <config.h>
 
@@ -47,7 +47,7 @@ void  rmJob::SayFinal()
    if(failed==file_count)
       return;
    if(file_count==1)
-      printf(_("%s ok, `%s' removed\n"),op,first->name);
+      printf(_("%s ok, `%s' removed\n"),op,first->name.get());
    else if(failed)
    {
       if(mode==FA::REMOVE_DIR)
