@@ -5,7 +5,7 @@
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
@@ -47,6 +47,9 @@ ParsedURL::ParsedURL(const char *url,bool proto_required,bool use_rfc1738)
 {
    parse(url,proto_required,use_rfc1738);
 }
+ParsedURL::~ParsedURL()
+{
+}
 
 void ParsedURL::parse(const char *url,bool proto_required,bool use_rfc1738)
 {
@@ -487,3 +490,9 @@ const char *url::remove_password(const char *url)
       return url;
    return xstring::format("%.*s%s",start-1,url,url+start+len);
 }
+
+bool url::is_url(const char *p)
+{
+   ParsedURL url(p,true);
+   return url.proto!=0;
+}
